package com.yaohun.aurechargedata.manage;

import com.yaohun.aurechargedata.Main;
import com.yaohun.aurechargedata.data.PlayerData;
import com.yaohun.aurechargedata.util.TimeType;
import com.yaohun.aurechargedata.util.UpdataTime;
import org.bukkit.Bukkit;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.entity.Player;
import org.bukkit.scheduler.BukkitRunnable;

import java.io.File;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

public class RechargeManage {

    private File file;
    private FileConfiguration yml;
    private String dataKey = "RechargeData";
    private HashMap<String, PlayerData> dataHashMap = new HashMap<>();

    public RechargeManage(){
        file = new File("plugins/AuRechargeData", "recharge.yml");
        // 如果文件不存在，则创建新文件
        if (!file.exists()) {
            try {
                file.getParentFile().mkdirs(); // 确保父文件夹存在
                file.createNewFile(); // 创建新文件
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        yml = YamlConfiguration.loadConfiguration(file);
        // 检查配置文件中是否包含 "RechargeData" 节点
        if (yml.getConfigurationSection(dataKey) != null) {
            boolean updated = false;  // 标记是否进行了数据更新
            if(UpdataTime.isDayDataUpdata()){
                // 遍历 "RechargeData" 节点下的所有键（用户名）
                for (String name : yml.getConfigurationSection(dataKey).getKeys(false)) {
                    yml.set(dataKey+"."+name+".daydata",null);// 将每位玩家的每日充值数据置为空
                }
                // 保存配置文件
                try {
                    yml.save(file);
                } catch (IOException e) {
                    e.printStackTrace();
                }
                updated = true;  // 标记数据已更新
            }
            if(UpdataTime.isMonthDataUpdata()){
                // 遍历 "RechargeData" 节点下的所有键（用户名）
                for (String name : yml.getConfigurationSection(dataKey).getKeys(false)) {
                    yml.set(dataKey+"."+name+".monthdata",null);// 将每位玩家的每日充值数据置为空
                }
                // 保存配置文件
                try {
                    yml.save(file);
                } catch (IOException e) {
                    e.printStackTrace();
                }
                updated = true;  // 标记数据已更新
            }
            // 如果数据已更新，异步定时任务重新加载数据
            if(updated){
                new BukkitRunnable() {
                    @Override
                    public void run() {
                        // 遍历 "RechargeData" 节点下的所有键（用户名）
                        for (String name : yml.getConfigurationSection(dataKey).getKeys(false)) {
                            // 获取每位玩家的配置节
                            ConfigurationSection section = yml.getConfigurationSection(dataKey + "." + name);
                            // 创建玩家数据对象并存入数据映射
                            dataHashMap.put(name, new PlayerData(name, section));
                        }
                        cancel();// 取消定时任务
                    }
                }.runTaskLaterAsynchronously(Main.plugin,20L);
            }else {
                // 数据未更新时直接加载数据
                // 遍历 "RechargeData" 节点下的所有键（用户名）
                for (String name : yml.getConfigurationSection(dataKey).getKeys(false)) {
                    // 获取每位玩家的配置节
                    ConfigurationSection section = yml.getConfigurationSection(dataKey + "." + name);
                    // 创建玩家数据对象并存入数据映射
                    dataHashMap.put(name, new PlayerData(name, section));
                }
            }
            upDataMonthlyRechargesTop();
        }
        Bukkit.getConsoleSender().sendMessage("§6[充值数据] §f已有充值记录的玩家: §a" + dataHashMap.size() + "名");
    }

    public void SaveData(){
        yml.set(dataKey,null);
        for (String name : dataHashMap.keySet()){
            PlayerData playerData = dataHashMap.get(name);
            for (TimeType timeType : playerData.getValueMap().keySet()){
                int value = playerData.getValueData(timeType);
                String timeKey = playerData.getTimeToString(timeType);
                if(value >= 1) {
                    yml.set(dataKey + "." + name + "." + timeKey, value);
                }
                Bukkit.getConsoleSender().sendMessage("[调试] "+name+" value: "+timeKey+" = "+value);
            }
            long lastOperation = playerData.getLastOperation();
            if(lastOperation >= 1000){
                yml.set(dataKey+"."+name+".lastOperation",lastOperation);
            }
        }
        // 保存配置文件
        try {
            yml.save(file);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public PlayerData getPlayerData(String name) {
        if(dataHashMap.get(name) == null){
            dataHashMap.put(name,new PlayerData(name,null));
            return dataHashMap.get(name);
        }
        return dataHashMap.get(name);
    }

    private LinkedHashMap<String, Integer> monthRechargeTop = new LinkedHashMap<>();
    // 获取每月玩家充值排行数据
    public void upDataMonthlyRechargesTop(){
        HashMap<String,Integer> hashMap = new HashMap<>();
        // 遍历 dataHashMap 中的所有玩家数据
        for (PlayerData playerData : dataHashMap.values()) {
            // 获取玩家名称和每月充值金额
            String playerName = playerData.getName();
            int monthlyRecharge = playerData.getValueData(TimeType.Monthly);
            // 创建 PlayerMonthlyRecharge 对象，并添加到列表中
            hashMap.put(playerName,monthlyRecharge);
        }
        this.monthRechargeTop = hashMap.entrySet()
                .stream()
                .sorted(Map.Entry.comparingByValue(Comparator.reverseOrder()))
                .collect(Collectors.toMap(
                        Map.Entry::getKey,
                        Map.Entry::getValue,
                        (e1, e2) -> e1,
                        LinkedHashMap::new
                ));
    }

    public String getMonthlyRankingName(int rank){
        List<String> playerNames = new ArrayList<>(monthRechargeTop.keySet());
        if (rank <= playerNames.size()) {
            String playerName = playerNames.get(rank - 1);
            int monthlyRecharge = monthRechargeTop.get(playerName);
            return playerName;
        } else {
            return "ERROR";
        }
    }
    public int getMonthlyRankingValue(int rank){
        List<String> playerNames = new ArrayList<>(monthRechargeTop.keySet());
        if (rank <= playerNames.size()) {
            String playerName = playerNames.get(rank - 1);
            return monthRechargeTop.get(playerName);
        } else {
            return 0;
        }
    }

    public String getRankingPlayer(int rank,String type){
        if(type.equalsIgnoreCase("name")){
            return getMonthlyRankingName(rank);
        }
        return ""+getMonthlyRankingValue(rank);
    }

}
